import sys
import polib
from django.core.management.base import BaseCommand
from tradukoj.models import Namespace


class Command(BaseCommand):
    # pylint: disable=C0301
    help = 'Show incompatible Translation Key for plain tree generation'
    exitcode = 0
    data = None

    def add_arguments(self, parser):
        parser.add_argument(
            '--pofile',
            dest='pofile',
            required=False,
            help='Test pofile instead database',
        )

    def handle(self, *args, **options):
        if 'pofile' in options:
            po_file = polib.pofile(options['pofile'])
            self.data = {"test": {}}
            for entry in po_file:
                self.test_text(entry.msgid)
            self.stdout.write(self.style.SUCCESS(f"Done {options['pofile']}"))
            sys.exit(self.exitcode)

        for namespace in Namespace.objects.all():
            self.stdout.write(f"Testing {namespace}...")
            self.data = {"test": {}}
            for translationkey in namespace.translation_keys.all():
                self.test_text(translationkey.text)
            self.stdout.write(self.style.SUCCESS(f"Done {namespace} test."))
        sys.exit(self.exitcode)

    def test_text(self, translationkey):
        if not translationkey:
            return
        if not translationkey.strip():
            return
        deep = translationkey.split('.')
        _current_node = self.data['test']
        for i, element in enumerate(deep):
            if i == (len(deep) - 1):
                if not isinstance(_current_node, dict):
                    self.stdout.write(
                        f"Error: The key `{element}` in `{translationkey}` tries to be an object but there are string on this path"
                    )
                    self.exitcode = 1
                    break
                if element in _current_node:
                    self.stdout.write(
                        f"Error: The key `{element}` in `{translationkey}` tries to be an string but there are object on this path"
                    )
                    self.exitcode = 1
                    break
                _current_node[element] = str(translationkey)
                break
            if not element in _current_node:
                _current_node[element] = {}

            if isinstance(_current_node[element], str):
                self.stdout.write(
                    f"Error: The key `{element}` in `{translationkey}` tries to be an object but there are string on this path"
                )
                self.exitcode = 1
                break

            if element not in _current_node:
                _current_node[element] = {}

            _current_node = _current_node[element]



# Generated by Django 2.0.4 on 2019-01-22 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0012_auto_20181219_1842'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gettextfile',
            name='file',
            field=models.FileField(upload_to='uploads/tradukoj'),
        ),
    ]



# Generated by Django 2.0.4 on 2018-05-16 09:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='translationkey',
            name='key',
        ),
        migrations.AddField(
            model_name='translationkey',
            name='text',
            field=models.CharField(default='hello from one test', max_length=255, unique=True, verbose_name='text key'),
            preserve_default=False,
        ),
    ]



from django.db.models.signals import post_save
from django.dispatch import receiver
from .models import GetTextFile


@receiver(post_save, sender=GetTextFile)
def process_file(instance=None, **_kwargs):
    instance.done = False
    if instance.process_file():
        instance.done = True



from django.conf.urls import url
from . import classviews

urlpatterns = [
    # LCRUD insurances
    url(r'^public/$', classviews.PublicTranslationList.as_view(),
        name="public_tradukoj_translation_list"),

    url(
        r'^public/(?P<langtag>[-\w]+)/(?P<namespace>[-\w]+)/(?P<key__text>.*)/$',
        classviews.PublicTranslationRetrieve.as_view(),
        name="public_tradukoj_translation_retrieve",
    ),


    url(r'^public/plain/$', classviews.PublicTranslationRawList.as_view(
    ), name="public_tradukoj_translation_plain_list"),


    url(r'^private/plain/$', classviews.PrivateTranslationRawList.as_view(
    ), name="private_tradukoj_translation_plain_list"),


    url(r'^private/$', classviews.PrivateTranslationList.as_view(),
        name="private_tradukoj_translation_list"),


    url(
        r'^private/(?P<langtag>[-\w]+)/(?P<namespace>[-\w]+)/(?P<key__text>.*)/$',
        classviews.PrivateTranslationRetrieve.as_view(),
        name="private_tradukoj_translation_retrieve",
    ),


    url(r'^available/$', classviews.EnabledLangs.as_view(),
        name="available_tradukoj_translation_list"),


    url(r'^bestlangtag/$', classviews.BestlangtagList.as_view(),
        name="bestlangtag_tradukoj_translation_list"),

]



# Generated by Django 2.0.4 on 2018-05-16 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0003_auto_20180516_1859'),
    ]

    operations = [
        migrations.AddField(
            model_name='bcp47',
            name='enabled',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='bcp47',
            name='name',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Lang name'),
        ),
    ]



from django.contrib import admin
from tradukoj import models


class TranslationKeyAdmin(admin.ModelAdmin):
    search_fields = ('text', )
    list_display = ('__str__', )


class TranslationAdmin(admin.ModelAdmin):
    search_fields = (
        'key__text',
        'small',
        'largue',
    )
    raw_id_fields = ('key', )


admin.site.register(models.TranslationKey, TranslationKeyAdmin)
admin.site.register(models.GetTextFile)
admin.site.register(models.Namespace)
admin.site.register(models.Translation, TranslationAdmin)
admin.site.register(models.BCP47)



from rest_framework.fields import Field, empty
from rest_framework.exceptions import NotFound
from tradukoj.models import TranslationKey
from tradukoj.serializers import TranslationKeyWithTranslationsSerializer


class TradukojKeyTextField(Field):
    def to_internal_value(self, data):
        try:
            key = TranslationKey.objects.get(text=data)
        except TranslationKey.DoesNotExist:
            raise NotFound(f"key text for {self.field_name} not found")

        return key

    def to_representation(self, value):
        if value:
            return value.text
        return None


class TradukojSerializedField(TranslationKeyWithTranslationsSerializer):

    fallback_langtag = None

    def __init__(self, instance=None, data=empty, **kwargs):
        if 'fallback_langtag' in kwargs:
            self.fallback_langtag = kwargs['fallback_langtag']
            del kwargs['fallback_langtag']
        super(TradukojSerializedField, self).__init__(instance, data, **kwargs)

    def to_representation(self, instance):
        if not instance:
            return {}

        representation = {'text': instance.text, 'translations': {}}
        if self.fallback_langtag:
            representation['fallback'] = None
        for translation in instance.translations.select_related('bcp47').all():
            representation['translations'][
                translation.bcp47.langtag] = translation.str_translation()
            if (self.fallback_langtag
                    and translation.bcp47.langtag == self.fallback_langtag):
                representation['fallback'] = translation.str_translation()

        return representation



# Generated by Django 2.0.4 on 2018-05-18 10:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0004_auto_20180516_1956'),
    ]

    operations = [
        migrations.AlterField(
            model_name='translationkey',
            name='namespace',
            field=models.CharField(default='global', max_length=255, verbose_name='namespace'),
        ),
    ]



# Generated by Django 2.0.4 on 2018-11-15 20:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0006_bcp47_direction'),
    ]

    operations = [
        migrations.AddField(
            model_name='bcp47',
            name='default',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.0.4 on 2018-12-13 12:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0007_bcp47_default'),
    ]

    operations = [
        migrations.CreateModel(
            name='GetTextFile',
            fields=[
                ('id',
                 models.AutoField(
                     auto_created=True,
                     primary_key=True,
                     serialize=False,
                     verbose_name='ID')),
                ('file', models.FileField(upload_to='')),
                ('file_type',
                 models.IntegerField(
                     choices=[(0, 'PO file'), (1, 'MO file')], default=0)),
                ('last_updated_date',
                 models.DateTimeField(
                     auto_now=True,
                     db_index=True,
                     verbose_name='Last updated date')),
                ('done', models.BooleanField(default=False)),
                ('done_with_errors', models.BooleanField(default=False)),
                ('log', models.TextField(max_length=1024, verbose_name='Log')),
                ('bcp47',
                 models.ForeignKey(
                     on_delete=django.db.models.deletion.CASCADE,
                     related_name='get_text_files',
                     to='tradukoj.BCP47',
                     verbose_name='Lang')),
            ],
        ),
        migrations.CreateModel(
            name='Namespace',
            fields=[
                ('id',
                 models.AutoField(
                     auto_created=True,
                     primary_key=True,
                     serialize=False,
                     verbose_name='ID')),
                ('text',
                 models.CharField(
                     max_length=255, unique=True, verbose_name='text key')),
            ],
        ),
        migrations.AlterField(
            model_name='translationkey',
            name='namespace',
            field=models.CharField(max_length=255),
        ),
        migrations.AlterField(
            model_name='translationkey',
            name='text',
            field=models.CharField(max_length=255, verbose_name='text key'),
        ),
        migrations.AddField(
            model_name='translationkey',
            name='new_namespace',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='translation_keys',
                to='tradukoj.Namespace'),
        ),
        migrations.AlterUniqueTogether(
            name='translationkey',
            unique_together={('namespace', 'text')},
        ),
        migrations.AddField(
            model_name='gettextfile',
            name='namespace',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='get_text_files',
                to='tradukoj.Namespace'),
        ),
    ]



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()

setup(
    name='django-tradukoj',
    python_requires='>=3.6.0',
    version='1.2.4',
    packages=find_packages(exclude=['tests*']),
    description='A django IETF\'s BCP 47 DB-based translation system',
    long_description=README,
    long_description_content_type='text/markdown',
    author='Develatio Technologies S.L.',
    author_email='contacto@develat.io',
    url='https://github.com/develatio/django-tradukoj/',
    license='BSD',
    install_requires=['Django>=2.0', 'langcodes >= 1.4.1'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Internationalization",
        "Topic :: Software Development :: Localization",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
)



# pylint: disable=W0212
import uuid
from django.db import models
from django.db.models.fields.related_descriptors import (
    ForwardOneToOneDescriptor,
    ReverseOneToOneDescriptor,
)
from django.db.models.fields.reverse_related import OneToOneRel
from django.utils.translation import gettext_lazy as _


class TradukojForwardOneToOneDescriptor(ForwardOneToOneDescriptor):
    def __set__(self, instance, value):
        # TODO: delete previous key on new key set?
        # try:
        #     related_object = self.field.get_cached_value(instance)
        # except KeyError:
        #     val = self.field.get_local_related_value(instance)
        #     if None in val:
        #         related_object = None
        #     else:
        #         related_object = self.get_object(instance)
        # if related_object:
        #     related_object.delete()

        if not value.text and value.auto_key_text is not None:
            value.text = (
                f"{instance._meta.app_label}_{instance._meta.model_name}_{self.field.name}_{uuid.uuid4().hex}"
            )
            value.save()

        return super().__set__(instance, value)


class OneToOneTradukojField(models.ForeignKey):

    # Field flags
    many_to_many = False
    many_to_one = False
    one_to_many = False
    one_to_one = True

    related_accessor_class = ReverseOneToOneDescriptor
    forward_related_accessor_class = TradukojForwardOneToOneDescriptor
    rel_class = OneToOneRel

    description = _("One-to-one tradukoj relationship")

    def __init__(self,
                 to="tradukoj.TranslationKey",
                 on_delete=models.PROTECT,
                 to_field=None,
                 **kwargs):
        kwargs['unique'] = True
        to = "tradukoj.TranslationKey"
        super().__init__(to, on_delete, to_field=None, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        if "unique" in kwargs:
            del kwargs['unique']
        return name, path, args, kwargs

    # pylint: disable=W0221
    def formfield(self, **kwargs):
        if self.remote_field.parent_link:
            return None
        return super().formfield(**kwargs)

    def save_form_data(self, instance, data):
        if isinstance(data, self.remote_field.model):
            setattr(instance, self.name, data)
        else:
            setattr(instance, self.attname, data)

    def _check_unique(self, **kwargs):
        # Override ForeignKey since check isn't applicable here.
        return []



# Generated by Django 2.0.4 on 2018-12-13 12:39

from django.db import migrations


def switch_namespace_field(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Namespace = apps.get_model('tradukoj', 'Namespace')
    TranslationKey = apps.get_model('tradukoj', 'TranslationKey')
    namespace, _created = Namespace.objects.get_or_create(text='global')
    for tranlsation_key in TranslationKey.objects.all():
        tranlsation_key.new_namespace = namespace
        tranlsation_key.text = "{0}.{1}".format(tranlsation_key.namespace,
                                                tranlsation_key.text)
        tranlsation_key.save()


class Migration(migrations.Migration):

    dependencies = [
        ('tradukoj', '0008_auto_20181213_1238'),
    ]

    operations = [
        migrations.RunPython(switch_namespace_field),
    ]
